unit Switch;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TSwitchState = (ss_On, ss_Off);

  TSwitch = class(TCustomControl)
  private
    { Private declarations }
    fState: TSwitchState;
    fChange: TNotifyEvent; //uivatelsky definovan Udlost

    procedure SetState(const NewState:TSwitchState);
    function  GetState: TSwitchState;

    procedure WMWINDOWPOSCHANGED(var Msg: TWMWindowPosChanged);
                message wm_WindowPosChanged; //zprva kterou komponenta obdr pi zmn jejho okna
    procedure CmEnableChanged(var Msg: TMessage);
                message cm_EnabledChanged;

  protected
    { Protected declarations }
    procedure Paint; override;
    procedure PaintSwitch; virtual;
    procedure MouseDown(Button: TMouseButton;
                        Shift: TShiftState; X, Y: Integer); override;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Switch : TSwitchState;
    procedure TurnOn;
    Procedure TurnOff;
  published
    { Published declarations }
    property State : TSwitchState
             read GetState write SetState;
    property width  default 32;
    property height default 48;
    property Left   default 100;
    property Top    default 100;
    property Hint;
    property ParentShowHint;
    property ShowHint;
    property Enabled;
    property Visible;
    property PopupMenu;

    property OnChange: TNotifyEvent
             read fChange write fChange;

    //property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property TabStop;
    property TabOrder;
  end;

procedure Register;

implementation

{$R switch.res}

const
  BmpHeight = 48;
  BmpWidth  = 32;

type
  TSwitchBitmap = class
  public
    Switch_Pics: TBitmap;    { Obrzek s vypnaem }
    Switch_List: TList;      { Seznam uchovvajc vechny vytvoen vypnae}
    constructor Create;
    destructor Destroy; override;
end;

var SwitchBitmap: TSwitchBitmap=nil;

constructor TSwitchBitmap.Create;
begin
  inherited Create;
  Switch_Pics := TBitmap.Create;
{ vytvoen seznamu, ve kterm jsou uchovny informace o vech vytvoench vypnach }
  Switch_List := TList.Create;
end; //of TSwitchBitmap.Create

destructor TSwitchBitmap.Destroy;
begin
  //zruen obrzku i seznamu
  Switch_Pics.Free;
  Switch_List.Free;
  inherited Destroy;
end;

constructor TSwitch.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
{ pokud jde o prvn vypna na formuli , vytvo se seznam obrzk}
  if SwitchBitmap = nil then SwitchBitmap := TSwitchBitmap.Create;
  SwitchBitmap.Switch_LIST.Add(Self); { pidn tto instance vypnae do seznamu}
  fState := ss_Off;
  ControlStyle := ControlStyle + [csOpaque]    //  + [csFramed] + [csReflector]
                               + [csFixedHeight] + [csFixedWidth];

  Width := BmpWidth;
  height:= BmpHeight;
end;   //of TSwitch.Create;

destructor TSwitch.Destroy;
begin
{ vymazn aktuln instance ze seznamu}
  with SwitchBitmap.Switch_LIST do
     Delete (IndexOf(Self));
{ pokud ji v seznamu nejsou dn dal vypnae, zrume cel seznam}
    if SwitchBitmap.Switch_LIST.Count = 0 then
    begin
      SwitchBitmap.Free; { Zruen seznamu}
      SwitchBitmap:=nil; { a nastaven ukazatele na Nil}
    end;
  inherited destroy;
end; //of TSwitch.Destroy


procedure TSwitch.WMWINDOWPOSCHANGED(var Msg: TWMWindowPosChanged);
var r:TRect;
begin
  r.Left:=Msg.WindowPos^.x;
  r.Top:=Msg.WindowPos^.y;
  r.Right:= r.Left+BmpWidth; r.Bottom:=r.Top + BmpHeight;
  UpdateBoundsRect(r);
end; //TSwitch.WMWINDOWPOSCHANGED

procedure TSwitch.SetState(const NewState : TSwitchState);
begin
 if not enabled then exit; //pokud nen komponenta aktivn konec

 if fState <> NewState then //pi zmn stavu
   begin
     fState:= NewState; //nastavit nov stav
     if csDesigning in ComponentState then //v dob nvrhu
        Invalidate //vyvolat pekreslen formule
     else
       begin
         PaintSwitch; //pekreslit komponentu
         if Assigned(fChange) then OnChange(Self); //vyvolat udlost OnChange
       end;
   end;
end; //TSwitch.SetState

function TSwitch.GetState: TSwitchState;
begin
  Result:=fState;
end; //TSwitch.GetState


function TSwitch.Switch : TSwitchState;
begin
 if not enabled then //pokud komponenta nen aktivn
 begin
   result :=fState; //vrtit stav a konec
   exit;
 end;

 if fState = ss_On then fState := ss_Off //zmna stavu
 else fState:=ss_On;

 PaintSwitch; //pekreslen vypnae
 if Assigned(fChange) then OnChange(Self); //vyvoln udlosti OnChange
 Result := fState; //vrtit stav
end; //TSwitch.Switch

Procedure TSwitch.TurnOn;
begin
  SetState(ss_On);
  invalidate;
end; //TSwitch.TurnOn

procedure TSwitch.TurnOff;
begin
  SetState(ss_Off);
  invalidate;
end; //TSwitch.TurnOff

procedure TSwitch.Paint;
begin
  PaintSwitch;
end; //TSwitch.Paint

procedure TSwitch.PaintSwitch;
begin
  if fState = ss_On then//v zvislosti na stavu nast ze zdroje sprvn obrzek
     begin
       if Enabled then SwitchBitmap.Switch_Pics.Handle:=
                       LoadBitmap(Hinstance,'TURNON')
       else SwitchBitmap.Switch_Pics.Handle:=
            LoadBitmap(Hinstance,'DISTURNON');
     end
  else
     begin
       if Enabled then SwitchBitmap.Switch_Pics.Handle:=
                       LoadBitmap(Hinstance,'TURNOFF')
       else SwitchBitmap.Switch_Pics.Handle:=
            LoadBitmap(Hinstance,'DISTURNOFF');
     end;

  Canvas.Draw(0,0,SwitchBitmap.Switch_Pics); //nakreslit naten obrzek
end; //TSwitch.PaintSwitch

procedure TSwitch.MouseDown(Button: TMouseButton;
                            Shift: TShiftState; X, Y: Integer);
begin
  Switch;       //pepnout stav
  invalidate;   //pekreslit komponentu
  SetFocus;
  inherited MouseDown(Button, Shift, X,Y);
end;


procedure Register;
begin
  RegisterComponents('Zanme programovat v Delphi', [TSwitch]);
end;

procedure TSwitch.CmEnableChanged(var Msg: TMessage);
begin
  Invalidate; // pi zmn vlastnosti Enabled pekreslit
end;

end.
